/* Toy.c */

/*
 * 3-D toy construction system.  
 * This program is in the public domain.
 *
 * Juan C. Torres
 * 12-10-99
 */


#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <GL/glut.h>


#ifndef M_PI
#          define M_PI 3.14159265
#endif

  int width, height;
int Wind2D, Wind3D, WindLis;	

static GLfloat view_rotx=20.0, view_roty=30.0, view_rotz=0.0;
static GLint brick1,brick2,brick3,cursor, XC,ZC;

static int  MAXHIGH = 0;
static GLfloat angle = 0.0, YC;

static GLuint limit;
static GLuint count = 1;

int Dr = 5.0, Tr=5.0;
int	dx=10.0, dy =0.0, dz=10.0;

   static GLfloat red[4] = {0.8, 0.1, 0.0, 1.0 };
   static GLfloat green[4] = {0.0, 0.8, 0.2, 1.0 };
   static GLfloat blue[4] = {0.2, 0.2, 1.0, 1.0 };
   static GLfloat yellow[4] = {1.0, 1.0, 0.0, 0.0 };
   static GLfloat gray[4] = {0.9, 0.9, 0.9, 1.0 };
   static GLfloat white[4] = {1.0, 1.0, 1.0, 1.0 };
   static GLfloat darkgray[4] = {0.3, 0.3, 0.3, 1.0 };

int SelectedColor = 2;


int Quality=0;  /* Rendering with smooth mode */

typedef struct {
	int tipo;   /* indice de tipo */
	int color;  /* indice de color */
	int X;
	int Z;
	int H;		/* Altura * 3 */
	int rot;     /* rotada? */
	} BrickList;

#define MaxBricks 98
static BrickList Model[1000];
static int NBricks=0;

typedef struct {
	int sizeX;
	int sizeZ;
	int sizeH;
	GLint DL[3];	/* Dysplay list */	
	} TypeList;

#define MaxTypes 20
static TypeList Types[MaxTypes];
static int NTypes=0;


int menu;
typedef enum{ QUIT, NOTHING, SETPUT, CLEAR, DOROTATE, ROTATECAMARA, 
		MOVECAMARA } menu_entries; 


int mode3D=ROTATECAMARA;


#define MaxColors 10
static GLfloat colour[MaxColors][4]= { 	{0.1, 0.1, 0.1, 1.0 }, /* Black*/
				{1.0, 1.0, 1.0, 1.0 }, /* White*/
				{1.0, 0.0, 0.0, 1.0 }, /* Red*/
				{0.0, 1.0, 0.0, 1.0 }, /* Green*/
				{0.0, 0.0, 1.0, 1.0 }, /* Blue*/
				{1.0, 1.0, 0.0, 1.0 }, /* Yellow*/
				{0.0, 1.0, 1.0, 1.0 }, /* Cian*/
				{0.3, 0.7, 0.7, 1.0 }, /* Brown*/
				{1.0, 0.0, 1.0, 1.0 }, /* Magenta*/
				{1.0, 0.6, 0.0, 1.0 }, /* Orange*/};
typedef enum {BLACK, WHITE, RED, GREEN, BLUE, YELLOW, CIAN, BROWN, MAGENTA,
		ORANGE} colores;

#define MapSize 20
static int HighMap[MapSize][MapSize];
int XP=0, ZP=0;	/* actual pointer */

int rotation = 0;


/* Selecci�n y edici�n */

int BrickSelected = 0;


void InitData(){
int i,j;
for(i=0;i<MapSize;i++)
	for(j=0;j<MapSize;j++)
		HighMap[i][j] = 1;
NBricks=0;
NTypes = 0;

}


static void GetLimits( int ib, int *xmin, int *xmax, int *zmin, int *zmax)
{
int i,j;

*xmin = Model[ib].X;
*zmin = Model[ib].Z;
if( Model[ib].rot ) {
	*xmax = *xmin - Types[Model[ib].tipo].sizeZ+1;
	*zmax = *zmin + Types[Model[ib].tipo].sizeX-1;
	}
else {	
	*xmax = *xmin + Types[Model[ib].tipo].sizeX-1;
	*zmax = *zmin + Types[Model[ib].tipo].sizeZ-1;
	}
}

void Dump() {

int k,i,j,xmin,xmax,zmin,zmax,imin,imax,jmin,jmax;
int Image[MapSize][MapSize];
int H[MapSize][MapSize];
for(i=0;i<MapSize;i++)
	for(j=0;j<MapSize;j++) {
		H[i][j] = 0;
		Image[i][j] = 0;
		}
for(k=0;k<NBricks; k++) {
	GetLimits( k, &xmin, &xmax, &zmin, &zmax);
	imin = xmin; jmin = zmin; imax = xmax; jmax = zmax;
	if(imin<0) imin = 0;
	if(jmin<0)  jmin = 0;
	if(imax>(MapSize-1)) imax = MapSize-1;
	if(jmax>(MapSize-1)) jmax = MapSize-1;
	for(i=imin;i<=imax;i++)
		for(j=jmin;j<=jmax;j++)
			if (Model[k].H>H[i][j]) {
				H[i][j] =  Model[k].H;
				Image[i][j] = k;
				}
	}
printf( "\n\n D U M P \n");
	for(i=0;i<MapSize;++i ) {
	     printf(" %d: ",i);
	     for(j=0;j<MapSize;++j ) {
		printf("%d ", Image[i][j]);
		}
	     printf("\n");
	    }
}



static void GetTypeLimits( int id, int x, int z, int rot, int *xmax, int *zmax){

if(rot ) {
	*xmax = x - Types[id].sizeZ+1;
	*zmax = z + Types[id].sizeX-1;
	}
else {	
	*xmax = x + Types[id].sizeX-1;
	*zmax = z + Types[id].sizeZ-1;
	}
}

static int GetHigh( xmin, xmax, zmin, zmax ){
int i,j,h,imin,imax,jmin,jmax;
h = 0;
imin = xmin; jmin = zmin; imax = xmax; jmax = zmax;
if(imin<0) imin = 0;
if(jmin<0)  jmin = 0;
if(imax>(MapSize-1)) imax = MapSize-1;
if(jmax>(MapSize-1)) jmax = MapSize-1;
for(i=imin;i<=imax;i++)
	for(j=jmin;j<=jmax;j++)
		if (h<HighMap[i][j]) h =  HighMap[i][j];
return h;
}



static void put( int id, int x, int z, int r, int col)
{
int xmax, zmax;
int i,j,h,imin,imax,jmin,jmax,k;

GetTypeLimits( id, x, z, r, &xmax, &zmax);

/*printf(">> %i: (%i,%i)-(%i,%i)  [%i]\n", id, x, z, xmax, zmax, r);*/


imin = x; jmin = z; imax = xmax; jmax = zmax;
if(imin>imax) {	
	k = imin;
	imin = imax;
	imax = k;
	} 
if(jmin>jmax) {	
	k = jmin;
	jmin = jmax;
	jmax = k;
	} 

h = GetHigh(imin,imax, jmin, jmax);
Model[NBricks].tipo = id;
Model[NBricks].color = col;
Model[NBricks].X = x;
Model[NBricks].Z = z;
Model[NBricks].H = h;
Model[NBricks].rot = r;
NBricks++;

if(imin<0) imin = 0;
if(jmin<0)  jmin = 0;
if(imax>(MapSize-1)) imax = MapSize-1;
if(jmax>(MapSize-1)) jmax = MapSize-1;
/*printf("check %i: (%i,%i)-(%i,%i)  [%i]\n", id, imin, jmin, imax, jmax, r);*/
for(i=imin;i<=imax;i++)
	for(j=jmin;j<=jmax;j++)
	 	HighMap[i][j] =  h + Types[id].sizeH;
if(MAXHIGH<(h+Types[id].sizeH)) MAXHIGH= h+Types[id].sizeH;

}

void AddBrick()  /* Add a random brick */
{
int id, x, z, r, c;

if( NBricks < MaxBricks) {
	id = 1+(int) ((1.0*(NTypes-1)*rand()) / (RAND_MAX+1.0));
	x  = 5.0 + (int) ((15.0*rand()) / (RAND_MAX+1.0));
	z  = 5.0 + (int) ((15.0*rand()) / (RAND_MAX+1.0));
	r  =  (int)( 0.5 + 1.0*rand() / (RAND_MAX+1.0)); 
	c  = 1+(int) (((MaxColors-1.0)*rand()) / (RAND_MAX+1.0));

	put(id, x, z, r, c);
/*	printf("\n Added %d : %d %d %d %d %d",NBricks,id,x,z,r,c); */
	}
glutSetWindow(Wind3D);
glutPostRedisplay();

glutSetWindow(Wind2D);
glutPostRedisplay();

}



static void BuildCursor3D()
{
   cursor = glGenLists(1);
   glNewList(cursor, GL_COMPILE);
   glColor3f( 1.0, 1.0, 0.0 );	  
   glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE,yellow  );

   	glBegin(GL_QUADS);
		glNormal3f( 0.0, 1.0, 0.0 );		
   		glVertex3f( 0.0, 0.1, 0.0);
		glVertex3f( 0.0, 0.1, 1.0);
		glVertex3f( 1.0, 0.1, 1.0);	
		glVertex3f( 1.0, 0.1, 0.0);
		glVertex3f( 0.0, 0.1, 0.0);
/*		glVertex3f( 0.0, 0.0, 1.0);
		glVertex3f( 1.0, 0.0, 1.0);
		glVertex3f( 1.0, 1.0, 1.0);
		glVertex3f( 0.0, 1.0, 1.0);
		glVertex3f( 0.0, 0.0, 1.0); */
	glEnd();
   glEndList();


}



/*
 * Draw a construction brick.  
 *
 * Input:  m = Z-size
 *         n = X-size
 * 	   h = 3* heigh         
 */
static void piece( GLint n, GLint m, GLint h)
{
   GLint i;
   GLfloat d0=0.25, d1=0.05;
   GLfloat hy;

   GLint j,k;
   GLfloat x,y,z,r,u,v;


   glShadeModel( GL_FLAT );  

 
   /* draw  lateral faces */
/*   glBegin( GL_QUAD_STRIP );
      hy = h/3.0;
 
      glNormal3f( -1.0, 0.0, 0.0 );
      glVertex3f( 0, hy,  0);
      glVertex3f( 0,  0,  0);
      glVertex3f( 0, hy,m);
      glVertex3f( 0, 0, m);

      glNormal3f( 0.0, 0.0, 1.0 );
      glVertex3f( n, hy,  m);
      glVertex3f( n,  0,  m);

      glNormal3f( 1.0, 0.0, 0.0 );
      glVertex3f( n,  hy, 0);
      glVertex3f( n,  0,  0);

      glNormal3f( 0.0, 0.0, -1.0 );
      glVertex3f( 0,  hy, 0);
      glVertex3f( 0,  0,  0);


       glEnd();
*/

   glBegin( GL_QUAD_STRIP );
      hy = h/3.0-d1;
 
     glNormal3f( 0.0, 0.0, -1.0 );
      glVertex3f( d1, hy,  0);
      glVertex3f( d1,  0,  0);

      glNormal3f( -1.0, 0.0, 0.0 );
      glVertex3f( 0, hy,  d1);
      glVertex3f( 0,  0,  d1);

      glVertex3f( 0, hy,  m-d1);
      glVertex3f( 0,  0,  m-d1);

      glNormal3f( 0.0, 0.0, 1.0 );
      glVertex3f( d1, hy,  m);
      glVertex3f( d1,  0,  m);
      glVertex3f( n-d1, hy,m);
      glVertex3f( n-d1, 0, m);

      glNormal3f( 1.0, 0.0, 0.0 );
      glVertex3f( n, hy,  m-d1);
      glVertex3f( n,  0,  m-d1);
      glVertex3f( n, hy,  d1);
      glVertex3f( n,  0,  d1);

      glNormal3f( 0.0, 0.0, -1.0 );
      glVertex3f( n-d1, hy,  0);
      glVertex3f( n-d1,  0,  0);
      glVertex3f( d1,  hy, 0);
      glVertex3f( d1,  0,  0);

       glEnd();
 
  /* draw  top faces */
/*   glBegin( GL_QUAD_STRIP );
   
      glNormal3f( 0.0, 1.0, 0.0 );
      glVertex3f( 0, hy,  0);
      glVertex3f( 0, hy,  m);
      glVertex3f( n, hy,  0);
      glVertex3f( n, hy,  m);

   glEnd();
*/

   glBegin( GL_QUAD_STRIP );
      hy = hy +d1;
      glNormal3f( -1.0, 0.0, 0.0 );
      glVertex3f( 0, hy-d1,  d1);
      glVertex3f( 0, hy-d1,  m-d1);

      glNormal3f( 0.0, 1.0, 0.0 );
      glVertex3f( d1, hy,  d1);
      glVertex3f( d1, hy,  m-d1);
      glVertex3f( n-d1, hy,  d1);
      glVertex3f( n-d1, hy,  m-d1);

      glNormal3f( 1.0, 0.0, 0.0 );
      glVertex3f( n, hy-d1,  d1);
      glVertex3f( n, hy-d1,  m-d1);
   glEnd();
   glBegin( GL_QUAD_STRIP );
      glNormal3f( -1.0, 0.0, 0.0 );
      glVertex3f( 0, hy-d1,  d1);
      glVertex3f( 0, hy-d1,  d1);
  
      glNormal3f( 0.0, 0.0, -1.0 );
      glVertex3f( d1, hy-d1,  0);

      glNormal3f( 0.0, 1.0, 0.0 );
      glVertex3f( d1,   hy,  d1);

      glNormal3f( 0.0, 0.0, -1.0 );
      glVertex3f( n-d1, hy-d1, 0);

      glNormal3f( 0.0, 1.0, 0.0 );
      glVertex3f( n-d1, hy,  d1);

      glNormal3f( 1.0, 0.0, 0.0 );
      glVertex3f( n, hy-d1,  d1);
      glVertex3f( n, hy-d1,  d1);
 
   glEnd();
   glBegin( GL_QUAD_STRIP );
      glNormal3f( -1.0, 0.0, 0.0 );
      glVertex3f( 0, hy-d1,  m-d1);
      glVertex3f( 0, hy-d1,  m-d1);

      glNormal3f( 0.0, 1.0, 0.0 );
      glVertex3f( d1,   hy,  m-d1);

      glNormal3f( 0.0, 0.0, 1.0 );
      glVertex3f( d1, hy-d1,  m);
 
      glNormal3f( 0.0, 1.0, 0.0 );
      glVertex3f( n-d1, hy,  m-d1);

      glNormal3f( 0.0, 0.0, 1.0 ); 
      glVertex3f( n-d1, hy-d1, m);
 


      glNormal3f( 1.0, 0.0, 0.0 );
      glVertex3f( n, hy-d1,  m-d1);
      glVertex3f( n, hy-d1,  m-d1);

   glEnd();

  
/*  Bottom faces */
    glNormal3f( 0.0, -1.0, 0.0 );

    glBegin( GL_QUAD_STRIP );
    glVertex3f( 0,  0,  0);
    glVertex3f( d0, 0,  d0);

    glVertex3f( 0, 0,  m);
    glVertex3f( d0, 0,  m-d0);

    glVertex3f( n, 0,  m);
    glVertex3f( n-d0, 0,  m-d0);

    glVertex3f( n,    0,  0);
    glVertex3f( n-d0, 0,  d0);

    glVertex3f( 0,   0,  0);
    glVertex3f( d0, 0,  d0);

   glEnd();

/* Interior */

   /*  lateral faces */   

   hy = h/3.0-d0;
   glBegin( GL_QUAD_STRIP );
     
        glNormal3f( 1.0, 0.0, 0.0 );
      glVertex3f(   d0,  0,  d0);
      glVertex3f(   d0, hy,  d0);

     glVertex3f( d0,  0,  m-d0);
     glVertex3f( d0, hy,  m-d0);

      glNormal3f( 0.0, 0.0, -1.0 );
      glVertex3f(   n-d0, 0, m-d0);
      glVertex3f(   n-d0,  hy,m-d0);


      glNormal3f( -1.0, 0.0, 0.0 );
       glVertex3f( n-d0,  0,  d0);
       glVertex3f( n-d0, hy,  d0);

    
      glNormal3f( 0.0, 0.0, 1.0 );
      glVertex3f( d0,  0,  d0);
      glVertex3f( d0, hy,  d0);

          glEnd();

   /* Interior top faces */
   glBegin( GL_QUAD_STRIP );
     glNormal3f( 0.0, -1.0, 0.0 );
     glVertex3f(   d0, hy,  m-d0 );
     glVertex3f(   d0, hy,  d0);

      glVertex3f( n-  d0, hy,  m-  d0);
      glVertex3f( n-  d0, hy,    d0);
   glEnd();

 
/* Assembling cylinders */
r = 0.5 - d0;
y =  h/3.0+ 0.25;

for ( i=0; i<m; i++) {
	for (j =0; j< n; j++) {
		x = 0.5+j;
		z = 0.5+i;
		glShadeModel( GL_FLAT);
		glNormal3f(0.0, 1.0, 0.0);
		glBegin(GL_QUAD_STRIP);
		for(k=0; k<=12; ++k) {
			glVertex3f(x+r*cos(k*M_PI/6), y, z+r*sin(k*M_PI/6));
			glVertex3f(x, y, z);
			}
	           glVertex3f(x+r, y, z);
		   glVertex3f(x, y, z);
		glEnd();

	if(Quality) glShadeModel( GL_SMOOTH);
		glBegin(GL_QUAD_STRIP);
		for(k=0; k<=12; ++k) {
	   	    u = cos(k*M_PI/6);
 		    v = sin(k*M_PI/6);
	    	    glNormal3f(u, 0.0, v);
			glVertex3f(x+r*u, y-0.25, z+r*v);
		    	glVertex3f(x+r*u, y, z+r*v);

		    }
		glVertex3f(x+r, y-0.25, z);
	    	glVertex3f(x+r, y, z);


		glEnd();
		}
	}


     
}

static void piece2D( GLint n, GLint m, GLint h)
{
   GLint i;
   GLfloat d0=0.25, d1=0.05;
   GLfloat hy;

   GLint j,k;
   GLfloat x,y,z,r,u,v;


   glShadeModel( GL_FLAT );  

 
 
  /* draw  top face */
   hy = h/3.0;
   glBegin( GL_QUAD_STRIP );
   
      glNormal3f( 0.0, 1.0, 0.0 );
      glVertex3f( 0, hy,  0);
      glVertex3f( 0, hy,  m);
      glVertex3f( n, hy,  0);
      glVertex3f( n, hy,  m);

   glEnd();


   
  
 
/* Assembling cylinders */
r = 0.5 - d0;
y =  h/3.0+ 0.25;

for ( i=0; i<m; i++) {
	for (j =0; j< n; j++) {
		x = 0.5+j;
		z = 0.5+i;
		glShadeModel( GL_FLAT);
		glNormal3f(1.0, 0.0, 1.0);
		glBegin(GL_QUAD_STRIP);
		for(k=0; k<=12; ++k) {
			glVertex3f(x+r*cos(k*M_PI/6), y, z+r*sin(k*M_PI/6));
			glVertex3f(x, y, z);
			}
	           glVertex3f(x+r, y, z);
		   glVertex3f(x, y, z);
		glEnd();

		}
	}


     
}



static void pick_brick(int button,int state,int x,int y)
{

switch ( button) 
	{
	case GLUT_LEFT_BUTTON:
		if( state == GLUT_DOWN) {
			BrickSelected = (480-y)/23.0;
/*			printf(" > %i, %i [%i]\n",x,y, BrickSelected);*/
			}
	}

glutSetWindow(WindLis);
glutPostRedisplay();
glutSetWindow(Wind3D);
glutPostRedisplay();
glutSetWindow(Wind2D);
glutPostRedisplay();

}
static void putBrick()
{
if(BrickSelected!=0) put(BrickSelected, XC, ZC, rotation, SelectedColor);
}


static void drawList( )
{
   int i;
   glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

/*for(i=0;i<NTypes;++i) printf("Lista: %i -> %i",i,Types[i].DL[2]); */

   glPushMatrix();
   for(i=1;i<NTypes;i++){
	glPushMatrix();
	    glTranslatef( -9.0, 20+4.0*i, -200.0);
            glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, gray);
	if(BrickSelected==i) glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE,colour[SelectedColor]);
	    glRotatef( 20.0, 1.0, 0.0, 0.0 );
	    glRotatef( 20.0, 0.0, 1.0, 0.0 );
	    glScalef(2.0,2.0,2.0);
	    glCallList(Types[i].DL[0]);   
/*printf("L: %i  \n",glIsList(Types[i].DL[2])); */
/*	printf("D: %i \n",Types[i].DL[2]); */
/*	    printf(".");  */
	glPopMatrix();
	}
/*printf("\n");*/

   glPopMatrix();
   glFinish();
   glutSwapBuffers();
}


static void axes()	/* Dibuja ejes */
{
glBegin(GL_LINES);{
  glColor3f(1.0,0.8,0.8);
  glVertex3f( 0, 0, 0 );
  glVertex3f( 0, 30, 0 );
  glVertex3f( 0, 0, 0 );
  glVertex3f( 30, 0, 0 );
  glVertex3f( 0, 0, 0 );
  glVertex3f( 0, 0, 30 );

	}
glEnd();
}

static void draw3D(  )
{
int i;

/*  printf(" %i-%i \n", NBricks, NTypes); */

/*drawList();*/

  glEnable( GL_DEPTH_TEST );


  glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

   glEnable( GL_CULL_FACE );
   glEnable( GL_LIGHTING );
   glEnable( GL_LIGHT0 );
   glEnable( GL_LIGHT1 );
   glEnable( GL_LIGHT2 );

/*if(NBricks==98) for(i=0;i<NTypes;++i) printf("Lista: %i -> %i",i,Types[i].DL[0]);
*/
   glPushMatrix();
   glTranslatef( dx, dy, dz);
   glRotatef( view_rotx, 1.0, 0.0, 0.0 );
   glRotatef( view_roty, 0.0, 1.0, 0.0 );
   glRotatef( view_rotz, 0.0, 0.0, 1.0 );
   glTranslatef( -10.0, 0.0, -10.0); 
/*   glTranslatef( dx, dy, dz-40.0);  */
 
  axes();

 /* piece(5,5,1); */

   for(i=0; i<NBricks; ++i ) 
 	{	
	glPushMatrix();
            glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, colour[Model[i].color] );
	    glMaterialfv( GL_FRONT, GL_SPECULAR, darkgray);
    	    glTranslatef( Model[i].X,Model[i].H/3.0, Model[i].Z ); 
	    if(Model[i].rot) {
		        glTranslatef( 0.5, 0.0, 0.5); 
			glRotatef(-90.0, 0.0, 1.0, 0.0);
		        glTranslatef( -0.5, 0.0, -0.5); 
			}
   	    glCallList(Types[Model[i].tipo].DL[0]);
/*	    glCallList(1);
if(NBricks==98) {
	printf("P:%i %i  \n",i,glIsList(Types[Model[i].tipo].DL[0]));
	
	}
	    piece(1,1,1); 
   	glPopMatrix();
	}

glCallList(Types[0].DL[0]);

   for(i=1;i<NTypes;i++){
	glPushMatrix();
	    glTranslatef( -1.0, 2.0*i, 2.0*i);
            glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, red);
	    glCallList(Types[i].DL[0]);
if(NBricks==98) {
	printf("LP:%i %i  \n",i,glIsList(Types[i].DL[0]));
	}
		    piece(1,1,1); */
	glPopMatrix();
	}

   if(BrickSelected!=0){
	    glTranslatef( XC, MAXHIGH/3.0+1.0, ZC );
            glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE,colour[SelectedColor] );
	    if(rotation) {
		        glTranslatef( 0.5, 0.0, 0.5); 
			glRotatef(-90.0, 0.0, 1.0, 0.0);
		        glTranslatef( -0.5, 0.0, -0.5); 
			}
	    glCallList(Types[BrickSelected].DL[0]);
	}
   glPopMatrix();


  glFinish();
  glutSwapBuffers();
}




static void draw2D( )
{
int i;

glEnable( GL_DEPTH_TEST );


glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

   glEnable( GL_CULL_FACE );
   glEnable( GL_LIGHTING );
   glEnable( GL_LIGHT0 );
   glEnable( GL_LIGHT1 );
   glEnable( GL_LIGHT2 );

   glPushMatrix();
   glTranslatef( 0.0,0.0,-30.0);
/*   glRotatef( 90., 1.0, 0.0, 0.0 );
   glRotatef( 0.0, 0.0, 1.0, 0.0 );
   glRotatef( 0.0, 0.0, 0.0, 1.0 ); */
   glTranslatef( -10.0, 0.0, -10.0); 
 
  axes();


   for(i=0; i<NBricks; ++i ) 
 	{	
	glPushMatrix();
            glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, colour[Model[i].color] );
    	    glTranslatef( Model[i].X,Model[i].H/3.0, Model[i].Z ); 
	    if(Model[i].rot) {
		        glTranslatef( 0.5, 0.0, 0.5); 
			glRotatef(-90.0, 0.0, 1.0, 0.0);
		        glTranslatef( -0.5, 0.0, -0.5); 
			}
   	    glCallList(Types[Model[i].tipo].DL[1]);

	glPopMatrix();
	}
   if(BrickSelected!=0){
	    glTranslatef( XC, MAXHIGH/3.0+1.0, ZC );
            glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE,colour[SelectedColor] );
	    if(rotation) {
		        glTranslatef( 0.5, 0.0, 0.5); 
			glRotatef(-90.0, 0.0, 1.0, 0.0);
		        glTranslatef( -0.5, 0.0, -0.5); 
			}
	    glCallList(Types[BrickSelected].DL[1]);
	}


   glPopMatrix();
   glutSwapBuffers();
}


static int Notdone=1;



static void idle( void )
{
int i,j;
  /* 
   count++;
   if (count>4) {}
	if( NBricks < MaxBricks/10) {AddBrick();}
*/
/*	else if(Notdone) {
	printf( "Colors \n");
	for(i=0;i<MaxColors;++i )
		printf(" %d: %f %f %f %f \n", i, 
		color[i][0],color[i][1],color[i][2],color[i][3]);
	printf( "Bricks \n");
	for(i=0;i<NTypes;++i )
		printf(" %d: %d %d %d %d \n", i, 
		Types[i].sizeX, Types[i].sizeZ,Types[i].sizeH,Types[i].DL[0]);
	printf( "Model \n");
	for(i=0;i<NBricks;++i )
		printf(" %d: %d %d %d %d %d %d\n", i, 
		Model[i].tipo,Model[i].color,Model[i].X,Model[i].Z,
		Model[i].H,Model[i].rot );
	printf( "HighMap \n");
	for(i=0;i<MapSize;++i ) {
	     printf(" %d: ",i);
	     for(j=0;j<MapSize;++j ) {
		printf("%d ", HighMap[i][j]);
		}
	     printf("\n");
	    }
	Dump();
	Notdone = 0;
	}
	*/
/*   
   glutSetWindow(Wind3D);
   glutPostRedisplay();
   glutSetWindow(Wind2D);
   glutPostRedisplay();

*/
}


void quit( void )
{
	exit(0);
}




void clear( void )
{

}


static void initLight( void )
{
   static GLfloat pos0[4] = {5.0, 20.0, 10.0, 0.0 };
   static GLfloat pos1[4] = { 0.0, 50.0, 10.0, 0.0 };
   static GLfloat pos2[4] = {0.0, 5.0, -10.0, 0.0 };

   glLightfv( GL_LIGHT0, GL_POSITION, pos0 );
   glLightfv( GL_LIGHT1, GL_POSITION, pos1 );
   glLightfv( GL_LIGHT2, GL_POSITION, pos2 );

   glEnable( GL_CULL_FACE );
   glEnable( GL_LIGHTING );
   glEnable( GL_LIGHT0 );
   glEnable( GL_LIGHT1 );
   glEnable( GL_LIGHT2 );

   glEnable( GL_DEPTH_TEST );

 

   glEnable( GL_NORMALIZE );
}


static void reshape2D(GLsizei w, GLsizei h)
{

    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    	glOrtho(-20, 20, -20*h/w, 20*h/w, -1000.0, 10000.0);
    	glMatrixMode(GL_MODELVIEW);
    	glLoadIdentity();
/*	set_screen_wh(w,h); */
	glTranslatef(-7,-30,0.0);
	glRotatef(90.0,1.0,0.0,0.0);
  	initLight();
glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

}

static void reshape3D(GLsizei w, GLsizei h)
{

    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    glFrustum( -0.5, 0.5, -h/(2.0*w), h/(2.0*w), 1, 100000.0 );
   	glMatrixMode(GL_MODELVIEW);
   	glLoadIdentity();
   	glTranslatef( 0.0, 0.0 , -100 );
/*
  glViewport(0, 0, (GLint)width, (GLint)height);
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if (w>h) {
      GLfloat w = (GLfloat) w / (GLfloat) h;
      glFrustum( -w, w, -1.0, 1.0, 2.0, 80.0 );
   }
   else {
      GLfloat h = (GLfloat) height / (GLfloat) width;
      glFrustum( -1.0, 1.0, -h, h, 2.0, 80.0 );
   }

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   glTranslatef( -10.0, 0.0, -50.0 );*/
   initLight();


}


static void reshapeList(GLsizei w, GLsizei h)
{

    glViewport(0, 0, w, h);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
   	glOrtho( -10., 10.0, 0.0, 100.0, 1, 100000.0 );
   	glMatrixMode(GL_MODELVIEW);
   	glLoadIdentity();
	glTranslatef(0.0,0.0,-10.0);
	initLight();

}

static void AddABrickType(int x, int z, int h)
{
GLint BrickList;

if(NTypes<MaxTypes) {
   /* make the bricks */
   BrickList = glGenLists(1);
   glNewList(BrickList, GL_COMPILE);
   	piece( x, z, h );
   glEndList();
   Types[NTypes].sizeX = x;
   Types[NTypes].sizeZ = z;
   Types[NTypes].sizeH = h;
   Types[NTypes].DL[0] = BrickList;
   NTypes++;
	}
else printf("Internal error: Brick types store overflow/n");
}

static void LoadBricks(int i)
/* Crea los DisplayLists para la ventana actual */

{
GLint BrickList;
int k;

for(k=0;k<NTypes;++k) {
   /* make the bricks */
   BrickList = glGenLists(1);
   glNewList(BrickList, GL_COMPILE);
   	if(i==1) piece2D( Types[k].sizeX, Types[k].sizeZ, Types[k].sizeH );
   	else piece( Types[k].sizeX, Types[k].sizeZ, Types[k].sizeH );
   glEndList();
   Types[k].DL[i] = BrickList;
   }
}


static void BuildBricks()
{

   /* make the bricks */
  	AddABrickType(20,20,1);
  	AddABrickType(1 , 1,3);
  	AddABrickType(2 , 1,3);
  	AddABrickType(3 , 1,3);
  	AddABrickType(4 , 1,3);
  	AddABrickType(8,  1,3);
  	AddABrickType(12, 1,3);
  	AddABrickType(2 , 2,3);
  	AddABrickType(3 , 2,3);
  	AddABrickType(4 , 2,3);
  	AddABrickType(6 , 2,3);
  	AddABrickType(1 , 1,1);
  	AddABrickType(2 , 1,1);
  	AddABrickType(3 , 1,1);
  	AddABrickType(4 , 1,1);
    	AddABrickType(2 , 2,1);
  	AddABrickType(3 , 2,1);
  	AddABrickType(4 , 2,1);
  	AddABrickType(6 , 2,1);

/*	glutSetWindow(WindLis);
	glutPostRedisplay();

	glutSetWindow(Wind3D);
	glutPostRedisplay();

	glutSetWindow(Wind2D);
	glutPostRedisplay();

*/
}

void menu_selected(int entry)    	/* MENU Principal*/
{

	switch(entry)
	{
		case CLEAR:
			clear();
			break;
		case SETPUT:
			putBrick();
			break;
		case DOROTATE:
			rotation = 1- rotation;
			break;

		case QUIT:
			quit();
			break;
	}
  	glutSetWindow( Wind3D );
  	glutPostRedisplay();

  	glutSetWindow( Wind2D );
  	glutPostRedisplay();

}

void menu3D(int entry)    	/* MENU  3D*/
{
	int Old3DMode=mode3D;
	switch(entry)
	{
		case ROTATECAMARA:
			mode3D = ROTATECAMARA;
			break;
		case MOVECAMARA:
			mode3D = MOVECAMARA;
			break;
	}
}

void menu_colores(int entry)    	/* MENU  de colores */
{
if(entry>=0 && entry <10 && entry != SelectedColor) {
	SelectedColor = entry;
	glutSetWindow(WindLis);
	glutPostRedisplay();
	glutSetWindow(Wind2D);
	glutPostRedisplay();
	glutSetWindow(Wind3D);
	glutPostRedisplay();
	}
}


void key_pressed(unsigned char key,int x,int y)
{
	switch(key)
	{
		case 'q':
		case 'Q':
			quit();
			break;
		case 'c':
		case 'C':
			clear();
			glutSetWindow( Wind2D );
  			glutPostRedisplay();
	
			break;
		case 'd':
		case 'D':
			Dump();
			break;
		case 's':
		case 'S':
			Quality = 1-Quality;
			break;
		case 'z':
    			if(mode3D==ROTATECAMARA) view_rotz += Dr;
			else      dz += Tr;
  			glutSetWindow( Wind3D );
  			glutPostRedisplay();
			
    			break;
  		case 'Z':
    			if(mode3D==ROTATECAMARA) view_rotz -= Dr;
			else      dz -= Tr;
  			glutSetWindow( Wind3D );
  			glutPostRedisplay();
    			break;

  		default:{}
	}
  return;
}



static void special(int k, int x, int y)
{
  switch (k) {
  case GLUT_KEY_UP:
    if(mode3D==ROTATECAMARA) view_rotx += Dr;
    else      dy -= Tr;
    break;
  case GLUT_KEY_DOWN:
    if(mode3D==ROTATECAMARA) view_rotx -= Dr;
    else      dy += Tr;
    break;
  case GLUT_KEY_LEFT:
    if(mode3D==ROTATECAMARA) view_roty += Dr;
    else      dx -= Tr;
    break;
  case GLUT_KEY_RIGHT:
    if(mode3D==ROTATECAMARA) view_roty -= Dr;
    else      dx += Tr;
    break;
  default:
    return;
  }
  	glutSetWindow( Wind3D );
  	glutPostRedisplay();

/*  glutSetWindow( Wind2D );
  glutPostRedisplay();  */
}


void key_2D(unsigned char key,int x,int y)
{
	switch(key)
	{
		case 'q':
		case 'Q':
			quit();
			break;
		case 'r':
 		case 'R':
			rotation = 1-rotation;
    			break;
		case 'p':
		case 'P':
		case 12:
		case 13:
			putBrick();
			break;
  		default:{}
	}
  	glutSetWindow( Wind3D );
  	glutPostRedisplay();

  	glutSetWindow( Wind2D );
  	glutPostRedisplay();

  return;
}



static void special2D(int k, int x, int y)
{
  switch (k) {
  case GLUT_KEY_UP:
    ZC -= 1;
    break;
  case GLUT_KEY_DOWN:
    ZC += 1;
    break;
  case GLUT_KEY_LEFT:
    XC -= 1;
    break;
  case GLUT_KEY_RIGHT:
    XC += 1;
    break;
  default:
    return;
  }
  	glutSetWindow( Wind3D );
  	glutPostRedisplay();

  glutSetWindow( Wind2D );
  glutPostRedisplay();  
}


void mouse_clicked(int button,int state,int x,int y)
{
	x-= x%10;		
	y-= y%10;
	switch(button)
	{
		case GLUT_LEFT_BUTTON:
			if(state==GLUT_DOWN)
/*				left_down(x,y);	/* a�ade v�rtice */
/*				else if(mode==PUT)
					put_object(x,y);  /*a�ade objeto */
			break;
		case GLUT_MIDDLE_BUTTON:	/* Cierra pol�gono */
			if(state==GLUT_DOWN)
/*				middle_down(x,y);  */
			break;
	}
}

void mouse_2D(int button,int state,int x,int y)
{
/*	XC = x%10;	*/	/* trunca coordenadas para ajustar al grid */
/*	ZC = y%10;	*/
}


static void initMenu( void )
{

 	glutSetWindow( Wind2D );
	/* DEFINICION DEL MENU */
	menu=glutCreateMenu(menu_selected);
	glutAddMenuEntry("clear",CLEAR);
	glutAddMenuEntry("  ",NOTHING);
	glutAddMenuEntry("put",SETPUT);
	glutAddMenuEntry("rotate",DOROTATE);
	glutAddMenuEntry("rotateCamara",ROTATECAMARA);
	glutAddMenuEntry("moveCamara",MOVECAMARA);
	glutAddMenuEntry("  ",NOTHING);
	glutAddMenuEntry("quit",QUIT);

	glutAttachMenu(GLUT_RIGHT_BUTTON);
	glutMouseFunc(mouse_2D);
	glutKeyboardFunc(key_2D);
 	glutSpecialFunc(special2D);

 	glutSetWindow( Wind3D );
	/* DEFINICION DEL MENU */
	menu=glutCreateMenu(menu3D);
	glutAddMenuEntry("rotateCamara",ROTATECAMARA);
	glutAddMenuEntry("moveCamara",MOVECAMARA);

	glutAttachMenu(GLUT_RIGHT_BUTTON);
	glutKeyboardFunc(key_pressed);
 	glutSpecialFunc(special);

 	glutSetWindow( WindLis );
	/* DEFINICION DEL MENU */
	menu=glutCreateMenu(menu_colores);
	glutAddMenuEntry("Negro",BLACK);
	glutAddMenuEntry("Blanco",WHITE);
	glutAddMenuEntry("Rojo",RED); 
	glutAddMenuEntry("Verde",GREEN); 
	glutAddMenuEntry("Azul",BLUE); 
	glutAddMenuEntry("Amarillo",YELLOW); 
	glutAddMenuEntry("Celeste",CIAN); 
	glutAddMenuEntry("Marr�n",BROWN); 
	glutAddMenuEntry("Morado",MAGENTA);
	glutAddMenuEntry("Naranja",ORANGE);

	glutAttachMenu(GLUT_RIGHT_BUTTON);
	glutMouseFunc(pick_brick);
	glutKeyboardFunc(key_pressed);
 	glutSpecialFunc(special);
}




int main( int argc, char *argv[] )
{


   InitData();

   glutInit(&argc, argv);
   glutInitDisplayMode (GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE);
   width = 600;
   height = 600;


   
   glutInitWindowSize (width,height);
   Wind3D = glutCreateWindow (" 3 D ");
   glutDisplayFunc(draw3D);
   glutReshapeFunc(reshape3D);
 
/*  Crea los display lists de las piezas almacenando sus identificadores
    en la columna i de la estructura Types. */
   
   BuildBricks();
   put(0, 0, 0, 0, 3);	
   BuildCursor3D();
   XC = 0; YC=0.3; ZC=0;  

 
   glutInitWindowSize (width,height-100);
   glutInitDisplayMode (GLUT_DEPTH | GLUT_DOUBLE | GLUT_RGB );
   Wind2D = glutCreateWindow (argv[0]);
   glutDisplayFunc(draw2D);
   glutReshapeFunc(reshape2D); 

   LoadBricks(1);   
   
   WindLis = glutCreateSubWindow (Wind2D,width-100,0,100,height);   
   glutDisplayFunc(drawList);
   glutReshapeFunc(reshapeList); 
 
   LoadBricks(2); 

initMenu();
  
  glutIdleFunc( idle);
   glutMainLoop();
   return 0;  

}
